/*
  Loads a screen dump from a PBM (portable bitmap) file.
  Supported resolutions: 720x256, 720x512

    loadScreenFromPBM -> Get image row from a PBM file
                         and copy it to the Roller RAM
*/

#include "pbmin.h"
#include "kernel.h"
#include "screen.h"
#include "buffered_io.h"
#include "mytypes.h"

#define BUF_SIZE 128

bool headerOk;
bool doubleHeight;
errorCode result;


/* Reads two bytes from the buffer. Reads a block from disc
   as soon as the buffer is read to the end.
 */
int readBufInt(void) {
    unsigned char b;
    int i;

    b = readBufferedByte();
    i = b << 8;
    b = readBufferedByte();
    i = i | b;
  
    return i;
}


/* Checks whether the next value from the buffer equals
   the byte in the parameter.
 */
bool equalsNextBufVal(unsigned char b) {
    return readBufferedByte() == b;
}


/* Verifies the 'magic number', which should be 'P4' at the beginning of
   the file.
 */
bool isMagicNumberValid(void) {
    bool result;

    result = equalsNextBufVal(0x50) && // 'P'
             equalsNextBufVal(0x34) && // '4'
             equalsNextBufVal(0x0A);   // LF
    if (buf_io_recs == 0)
        result = false; // out of data
  
    return result;
}


/* Skips a comment line. If none was found, false is returned.
 */
void skipCommentLine() {
    bool result;
    unsigned char b;
    bool readMore;

    result = false;
    b = readBufferedByte();
    if (buf_io_recs > 0) {
        if (b == 0x23) { // '#'
            result = true;     // comment line found
            readMore = true;
            while (readMore) {
                readMore = !equalsNextBufVal(0x0A);  // LF = end of comment line
                if (buf_io_recs == 0) { // out of data
                    readMore = false;
                    result = false;
                }
            }
        }
        else {
            // set back buffer index, so the byte gets read again
            --buf_io_fbufidx;
        }
    }
    
    return result;
}


/* Reads and returns a resolution value (X or Y).
 */
int readRes(void) {
    unsigned char b;
    bool readMore;
    int res;
    int digitCount;

    res = 0;
    digitCount = 0;
    readMore = true;

    while (readMore) {
        b = readBufferedByte();
        if (buf_io_recs == 0) {
            readMore = false; // out of data
        }
        else {
            if (b == 0x20 || b == 0x0A) {  // ' ' or LF
                readMore = false;
            }
            else {
                res = 10 * res + b - '0';     // build the number from digits
                ++digitCount;
                if (digitCount > 3) {         // resolutions with more than
                    res = 0;                  // three digits are not supported
                    readMore = false;
                }
            }
        }
    }
  
    return res;
}


/* Reads a line of pixels. If invert is true, then all pixels get inverted.
 */
void readPixelLine(unsigned char y, bool invert) {
    unsigned char x;
    int i;
    unsigned char b;

    i = y;
    for (x = 0; x < 90; x++) { // 90 = 720 / 8
        b = readBufferedByte();
        if (invert) b = b ^ 0xFF; // XOR
        gx_buffer[i] = b;
        i += 8;
    }
}


/* Reads a line of pixels. If invert is true, then all pixels get inverted.
 */
void readPixelLineInt(unsigned char y, bool invert) {
    unsigned char x;
    int i;
    int data;
    unsigned char b;

    i = y;
    x = 0;
    while (x < 90) {
        data = readBufInt();
        if (invert) data = data ^ 0xFFFF; // XOR
        b = data >> 8;   // hi byte
        gx_buffer[i] = b;
        i += 8;
        b = data & 0xFF; // lo byte
        gx_buffer[i] = b;
        i += 8;
        ++x;
        ++x;
    }
}


/* Loads a screen dump from a black and white 1 bit per pixel PBM file.
 */
errorCode loadScreenFromPBM(char *filename, bool invert) {
    unsigned char row;
    unsigned char x, y;
    int i;
    unsigned char b;
    int resX;
    int resY;
    errorCode_t err;

    headerOk = false;
    doubleHeight = false;

    err = openBufferedFile(filename, "rb");
    if (err != ErrOk) {
        result = ErrFileNotFound;
    }
    else {
        // Read header
        if (isMagicNumberValid()) {
            while (skipCommentLine()) {
            }
            resX = readRes();
            if (resX > 0) resY = readRes();
            if (resY > 0) {
                if (resX <= 720 && resY <= 512) {
                    headerOk = true;
                    if (resX == 720 && resY == 512) doubleHeight = true;
                }
            }
        }

// TODO >>>
        if (headerOk) {
            // Read pixel data
            for (row = 0; row < 32; row++) {
                for (y = 0; y < 8; y++) {
/*
                    ReadPixelLine(y, invert);
                    if doubleHeight then ReadPixelLine(y, invert);
*/
                    readPixelLineInt(y, invert);
                    if (doubleHeight) readPixelLineInt(y, invert);
                }
                gx_putScreenRow(row); // puts 720 bytes = 8 rows of 90 bytes
            }
            result = ErrOk;
        }
        else {
            result = ErrFormatNotSupported;
        }
        
        closeBufferedFile();
    }
  
    return result;
}
